

#include <math.h>
#include <stdio.h>
#include "typedefs.h"
#include "dtmf.h"

/* DTMF Global */
u08  DTMFdigit, DTMFearly, DTMFbufferReady;

/* Ping Pong buffers for storing the DTMF samples */
s32 DTMFsample1[DTMF_NPOINTS];
s32 DTMFsample2[DTMF_NPOINTS];

extern s32 *p_DTMFsample;          		/* Current pointer to the DTMF sample  - Ping*/
extern s32 *p_DTMFend;					  			/* End pointer to the wave sample */

#ifndef _DEBUG
extern u08 dspmode;
#else
u08 dspmode;
s32 *p_DTMFsample;          		/* Current pointer to the DTMF sample  - Ping*/
s32 *p_DTMFend;					  			/* End pointer to the wave sample */
#endif

/* Coefficient: 2^16 * cos (2*PI*Fdtmf/Fsampling) */
/* Increasing frequency order, lower group first followed by high group */
static int DTMFCoeff[NDTMFCOEFF] = {  55959, 53913, 51403, 48438,
                                      38145, 32649, 26169, 18630 };

/* Valid combinations have a single bit set in the low and high groups */
/* High nibble is high group, Lower nibble is low group */
static int DTMFvalid[NDIGIT+1] = {	0x00,
                                    0x11, 0x21, 0x41, 0x81, 0x12, 0x22, 0x42, 0x82,
														  			0x14, 0x24, 0x44, 0x84, 0x18, 0x28, 0x48, 0x88 };
 
static int DTMFvalue[NDIGIT+1] = { NODTMF,
                                   '1', '2', '3', 'A', '4', '5', '6', 'B',
                                   '7', '8', '9', 'C', '*', '0', '#', 'D' };

/* Coefficient: 2^16 * cos (2*PI*Ftone/Fsampling) */
static int TONECoeff[NTONECOEFF] = { 63075, 61661 };

#ifdef _DEBUG

static int DTMFLow[4]	 = {  697,  770,  852,  941 };
static int DTMFHigh[4] = { 1209, 1336, 1477, 1633 };

void DTMFinit( int low, int high )
{
	double f1, f2, sum;
	int i;

	for (i=0; i<DTMF_NPOINTS; i++)
	{
		f1 = sin(2 * PI * i * DTMFLow[low]   / SAMPLINGRATE);
		f2 = sin(2 * PI * i * DTMFHigh[high] / SAMPLINGRATE);
		sum = 256 * (f1 + f2)/2;
		DTMFsample1[i]= (s32) sum;
	}
}

int main (void)
{
	int i, j;
	u08 decoded;

	for (i=0; i< 4; i++)
	{
		for (j=0; j<4; j++)
		{
			DTMFinit(i, j);
			decoded = DTMFprocess(&DTMFsample1[0]);
			if (decoded != NODTMF)
			{
				printf("Decoded (should not happen): %c", decoded);
			}
			decoded = DTMFprocess(&DTMFsample1[0]);
			if (decoded != NODTMF)
			{
				printf("Decoded: %c\n", decoded);
			}
		}
	}

	return 0;
}

#endif

/******************************************************************************
 * Function:        u08 DTMFprocess( s32 *DTMFsamples )
 * PreCondition:    None
 * Input:           DTMFsamples points to a buffer of DTMFnpoints samples
 * Output:          DTMFearly  is a DTMG digit or NODTMF
 *                  DTMFdetect is a DTMG digit or NODTMF
 * Side Effects:    None
 *
 * Overview:        Implements the Goertzel algorithm 
 * Note:            Inspired from an old version of the Asterisk PBX
 *****************************************************************************/

u08 DTMFprocess( s32 *DTMFsamples )
{

  int k, n;
  s32 sk, sk1, sk2;
	s32 power;
  u08 result = 0;

	/* Process the 8 frequency filters */
 	for (k = 0; k < NDTMFCOEFF; k++)
  {
	  /* Goertzel processing */
    sk = sk1 = sk2 = 0;
		for (n = 0; n < DTMF_NPOINTS; n++)
    {
			sk = DTMFsamples[n] + ((DTMFCoeff[k] * sk1) >> 15) - sk2;
			sk2 = sk1;
			sk1 = sk;
		}
		/* Prevent overflows */
		sk  >>= 1;
		sk2 >>= 1;
	
		/* compute |X(k)|**2 */
		power =	((sk * sk) >> AMP_BITS) -
		        ((((DTMFCoeff[k] * sk) >> 15) * sk2) >> AMP_BITS) +
		        ((sk2 * sk2) >> AMP_BITS);
		result = result >> 1;
		if (power > DTMF_TRESH)
		{
			result += 0x80;
		}
	}
	/* Test the result against the 16+1 valid combinations */
	for (k=0; k < NDIGIT+1; k++)
	{
		if ( result == DTMFvalid[k] )
		{
      /* Check if already detected that digit */
      if (DTMFvalue[k] != DTMFdigit )
      { 
        /* Wait for 2 consecutive detections */
			  if (DTMFvalue[k] == DTMFearly )
			  {
				  DTMFdigit = DTMFvalue[k];
			  } else {
				  DTMFdigit = NODTMF;
			  }
			  DTMFearly = DTMFvalue[k];
			  return DTMFdigit;
      }
		}
	}
  return NODTMF;
}




#ifndef _DEBUG
void DTMFactivate( void )
{
  p_DTMFsample = &DTMFsample1[0];											/* Init pointer */
	p_DTMFend = &DTMFsample1[0] + DTMF_NPOINTS;					/* Init end */
	DTMFbufferReady = 0;																/* Buffer not ready yet */
  DTMFearly = NODTMF;
  DTMFdigit = NODTMF;
  dspmode = DTMFDETECT;																/* Change the DSP task */
}
#endif

u08 DTMFdecode(void)
{
  u08 digit = NODTMF;

  if (DTMFbufferReady	!= 0)
	{
	  if (DTMFbufferReady  == 1)
		{
      digit = DTMFprocess( &DTMFsample1[0] );
	  } else {
      digit = DTMFprocess( &DTMFsample2[0] );
	  }
	  DTMFbufferReady = 0;
  }
  if (digit != NODTMF)
  {
      dspmode = IDLE;																			/* Back to idle mode */
  }
  return digit;
}

/******************************************************************************
 * Function:        u08 result DialToneprocess( s32 *ToneSamples )
 * PreCondition:    None
 * Input:           ToneSamples points to a buffer of DTMFnpoints samples
 * Output:          2 if a dial tone is detected
 *                  1 only one tone is detected
 *                  0 if nothing is detected
 * Side Effects:    None
 *
 * Overview:        Implements the Goertzel algorithm 
 * Note:            Inspired from an old version of the Asterisk PBX
 *****************************************************************************/

u08 DialToneprocess( s32 *ToneSamples )
{
  int k, n;
  s32 sk, sk1, sk2;
	u32 power;
  u08 result = 0;

	/* Process the 2 frequency filters */
 	for (k = 0; k < NTONECOEFF; k++)
  {
	  /* Goertzel processing */
    sk = sk1 = sk2 = 0;
		for (n = 0; n < TONE_NPOINTS; n++)
    {
			sk = ToneSamples[n] + ((TONECoeff[k] * sk1) >> 15) - sk2;
			sk2 = sk1;
			sk1 = sk;
		}
		/* Prevent overflows */
		sk  >>= 1;
		sk2 >>= 1;
	
		/* compute |X(k)|**2 */
		power =	((sk * sk) >> AMP_BITS) -
		        ((((TONECoeff[k] * sk) >> 15) * sk2) >> AMP_BITS) +
		        ((sk2 * sk2) >> AMP_BITS);
		if (power > TONE_TRESH)
		{
			result += 1;
		} else {
    }
	}
  return result;
}

void TONEactivate( void )
{
  p_DTMFsample = &DTMFsample1[0];											/* Init pointer */
	p_DTMFend = &DTMFsample1[0] + DTMF_NPOINTS;					/* Init end */
	DTMFbufferReady = 0;																/* Buffer not ready yet */
}

s08 TONEdecode(void)
{
  s08 result;

  result = -1;
  if (DTMFbufferReady	!= 0)
	{
	  if (DTMFbufferReady  == 1)
		{
      result = DialToneprocess( &DTMFsample1[0] );
	  } else {
      result = DialToneprocess( &DTMFsample2[0] );
	  }
	  DTMFbufferReady = 0;
  }
  return result;
}
