/*****************************************************************************
//
// File Name	  : 'enc38j60.c'
// Title		    : Driver for the Microchip ENC28J60 Ethernet controller
// Author		    : Bernard Debbasch - Copyright (C) 2005
// Created		  : 2005.08.17
// Revised		  : 2005.08.17
// Version		  : 1.0
// Target MCU	  : Philips LPC213x series
// Editor Tabs	: 2
//
//
//
// Copyright (C) 2005, Bernard Debbasch
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met: 
// 1. Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer. 
// 2. Redistributions in binary form must reproduce the above copyright 
//    notice, this list of conditions and the following disclaimer in the  
//    documentation and/or other materials provided with the distribution. 
// 3. The name of the author may not be used to endorse or promote
//    products derived from this software without specific prior
//    written permission.  
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
*******************************************************************************/

#include <LPC213x.H>                       /* LPC213x definitions  */
#include <stdio.h>
#include "typedefs.h"
#include "enc28j60.h"
#include "spiLPC.h"
#include "config.h"
#include "uipopt.h"
#include "uipopt.h"

//#define ETHERNET_DEBUG                  /* Define if trying to debug the driver */


/* RX & TX memory organization */
#define MAXFRAMESIZE 1518
#define RAMSIZE	0x2000		
#define TXSTART	0x0000
#define TXEND   0x0fff
#define RXSTART 0x1000
#define RXEND   0x1FFF
#define RXSIZE  (RXSTOP - RXSTART + 1)

/* Chose one or the other */
//#define HALF_DUPLEX
#define FULL_DUPLEX

#ifdef HALF_DUPLEX
#ifdef FULL_DUPLEX
DUPLEX==> CHOOSE ONE OR THE OTHER
#endif
#endif

/* Local Prototypes */
static u08 encReadEthReg (u08 address);
static u08 encReadMacReg (u08 address);
static void encWriteReg (u08 address, u08 data);
static void encWriteReg16(u08 address, u16 data);
static void encBFSReg(u08 address, u08 data);
static void encBFCReg(u08 address, u08 data);
static void SendSystemCommand (void);
static void encBankSelect(u08 bank);
static void encWritePHYReg(u08 address, u16 data);
static u16 encReadPHYReg(u08 address);
static void encMACwrite(u08 data);
static void encMACwriteBulk(u08 *buffer, u16 length);
static u08 encMACread(void);
static void encMACreadBulk(u08 *buffer, u16 length);

/* Global variable */
u16 ethRxPointer;                         // Point to the next available frame

extern volatile u16 Timer10ms_1;
extern volatile u16 uip_len, uip_slen;
extern u08 uip_buf[];
extern u08 *uip_appdata;          
extern struct uip_eth_addr uip_ethaddr;

/******************************************************************************
 * Function:        void EthernetInit (void)
 * PreCondition:    SPI must be configured
 * Input:           None
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Initialize the ENC28J60 
 * Note:            None
 *****************************************************************************/
void ethernetInit (void)
{
  u08 value;
  u16 value16;

  ETH_RESET;
  Timer10ms_1 = 10;
  while (Timer10ms_1 != 0) {};
  ETH_NORESET;

  /* Make sure the part is out of reset.
  Monitor CLKRDY bit until set. */
	do {
		value = encReadEthReg(ESTAT);
	} while (!(value & ESTAT_CLKRDY));

	/* Send a Soft Reset to the chip */
  SendSystemCommand();	
  Timer10ms_1 = 2;
  while (Timer10ms_1 != 0) {};  

  /* Start in Bank 0 to set the pointers */
  encBankSelect(BANK0);

  /* Initialize the local RX pointer and configure the base address */
  ethRxPointer = RXSTART;
  encWriteReg16(ERXSTL, RXSTART);
  /* Read pointer is also at the base address for now */
  encWriteReg16(ERXRDPTL, RXSTART);
  /* Configure the H/W with the end of memory location */
  encWriteReg16(ERXNDL, RXEND);

  /* Configure the H/W with the TX base address */
	encWriteReg16(ETXSTL, TXSTART);

  /* Go to Bank 1 */
  encBankSelect(BANK1);
	encWriteReg16(ERXFCON, ERXFCON_UCEN | ERXFCON_CRCEN | ERXFCON_BCEN);

   /* Go to Bank 2 */
  encBankSelect(BANK2);
 	/* Remove all reset conditions */
  encWriteReg(MACON2, 0);

	#ifdef HALF_DUPLEX
  /* Enable RX */
  encWriteReg(MACON1, MACON1_MARXEN);
	/* Automatic padding, CRC generation */
  encWriteReg(MACON3, MACON3_PADCFG0 | MACON3_TXCRCEN);
  #else
  /* Enable TX pause, RX pause and RX */
  encWriteReg(MACON1, MACON1_TXPAUS | MACON1_RXPAUS | MACON1_MARXEN);
  /* Automatic padding, CRC generation, Full-Duplex */
  encWriteReg(MACON3, MACON3_PADCFG0 | MACON3_TXCRCEN | MACON3_FULDPX);
	#endif

 	/* Set the maximum packet size which the controller will accept */
	encWriteReg16(MAMXFLL, MAXFRAMESIZE);
  
  /* Set inter-packet gap to 9.6us */
	encWriteReg16(MAIPGL, 0x0C12);

  /* Set back to back inter-packet gap */
	encWriteReg(MABBIPG, 0x12);

  /* Go to Bank 3 */
  encBankSelect(BANK3);
  encWriteReg(MAADR0, UIP_ETHADDR5);
  encWriteReg(MAADR1, UIP_ETHADDR4);
  encWriteReg(MAADR2, UIP_ETHADDR3);
  encWriteReg(MAADR3, UIP_ETHADDR2);
  encWriteReg(MAADR4, UIP_ETHADDR1);
  encWriteReg(MAADR5, UIP_ETHADDR0);

  #ifdef HALF_DUPLEX
  encWritePHYReg(PHCON1, 0);           
  #else
  encWritePHYReg(PHCON1, PHCON1_PDPXMD);
  #endif           
  
  /* Disable half duplex loopback in PHY. */
  encWritePHYReg(PHCON2, PHCON2_HDLDIS);           

	/* We use the default setting of the LED */

  /* Read a PHY register (so the compiler does not complain !) */
  value16 = encReadPHYReg(PHID1);
	/* Enable packet reception */
  encBFSReg(ECON1, ECON1_RXEN);
}

void ethernetSend (void)
{
  u16 length;
  u16 value;

  /* Save length for later */
  length = uip_len;
  /* Clear the end TX flag */
  encBFCReg(EIR, EIR_TXIF);

  /* Go to Bank 0 */
  encBankSelect(BANK0);
  /* Configure the H/W with the TX base address */
	encWriteReg16(ETXSTL, TXSTART);
  /* Configure the H/W with the TX Start and End addresses */
	encWriteReg16(EWRPTL, TXSTART);
  /* encWriteReg16(ETXNDL, TXSTART + uip_len + 1); */
  /* Write the per packet option byte
  we use the default values set in chip earlier */
	encMACwrite(0x00);

  /* Send 40+14=54 bytes of header */
  encMACwriteBulk(&uip_buf[0], 54);

  #ifdef ETHERNETDEBUG
  for (value=0; value< 54; value++) {
    printf ("%x ",uip_buf[value]);
  }
  printf("\n");
  #endif

  if (uip_len > 54) {
    /* Send the rest of the packet, the application data */
    uip_len -= 54;
    encMACwriteBulk(&uip_appdata[0], uip_len);
  }
  /* Configure the H/W with the TX Start and End addresses */
  encWriteReg16(ETXNDL, (TXSTART + length));
  value = encReadEthReg(ETXNDL);
  value += (encReadEthReg(ETXNDH)<<8);
	/* Enable packet Transmission */
  encBFSReg(ECON1, ECON1_TXRTS);
}

u16 ethernetReceive (void)
{
  u16 len, u;

	/* Check if at least one packet has been received and is waiting */
  if ( (encReadEthReg(EIR) & EIR_PKTIF) == 0) {
    return 0;
  }
  /* Set read Pointer */
  encBankSelect(BANK0);
  encWriteReg16(ERDPTL, ethRxPointer);

  ethRxPointer = encMACread();
  ethRxPointer += encMACread() << 8;

  /* Error detection... */
  if (ethRxPointer > RXEND) {
    ethernetInit();
    return 0;
  }

  len = encMACread();
  len += encMACread() << 8;

  /* Read 2 more statuses that we ignore */
  encMACread();
  encMACread();

  /* If the frame is too big to handle, throw it away */
  if (len > UIP_BUFSIZE) {
    for (u=0; u<len; u++) {
      encMACread();                    /* Read a byte and throw it away */
    } 
    return 0;
  }
  
  /* Read the whole frame */
  encMACreadBulk(&uip_buf[0], len);

  /* Clean up for next packet 
  Set the read pointer to the start of RX packet to free up
  space used by the current frame */
  /* Go to Bank 0 */
  encBankSelect(BANK0);
  encWriteReg16(ERXRDPTL, ethRxPointer);

  /* Decrement EPKTCNT */
  encBFSReg(ECON2, ECON2_PKTDEC);

  /* Return the length - the 4 bytes of CRC */
  return (len-4);  
}


/******************************************************************************
 * Function:        void SendSystemCommand(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    All values are lost
 *
 * Overview:        Send the SC (Reset) command to the device 
 * Note:            None
 *****************************************************************************/
static void SendSystemCommand(void)
{
  /* Select the chip and send the System Command (Reset) */
	SPI_SELECT_ETHERNET;
	spiPut(SC);	
	SPI_UNSELECT_ETHERNET;
}

/******************************************************************************
 * Function:        u08 encReadEthReg(u08 address)
 * PreCondition:    None
 * Input:           address to read
 * Output:          value read from the register
 * Side Effects:    None
 *
 * Overview:        read the value at the address over the SPI bus
 * Note:            None
 *****************************************************************************/
static u08 encReadEthReg(u08 address)
{
	u08 value;

  SPI_SELECT_ETHERNET;
	spiPut(RCR | address);	
  value = spiPut(0x00);							
	SPI_UNSELECT_ETHERNET;
	return value;
}

/******************************************************************************
 * Function:        u08 encReadMacReg(u08 address)
 * PreCondition:    None
 * Input:           address to read
 * Output:          value read from the register
 * Side Effects:    None
 *
 * Overview:        read the value at the address over the SPI bus
 * Note:            None
 *****************************************************************************/
static u08 encReadMacReg(u08 address)
{
	u08 value;

  SPI_SELECT_ETHERNET;
	spiPut(RCR | address);	
	spiPut(0x00);             /* Send a dummy byte */
  value = spiPut(0x00);							
	SPI_UNSELECT_ETHERNET;
	return value;
}

/******************************************************************************
 * Function:        void encWriteReg(u08 address, u08 data)
 * PreCondition:    Bank must point ot the right bank
 * Input:           address and data to write
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Send value at the address over the SPI bus
 * Note:            None
 *****************************************************************************/
static void encWriteReg(u08 address, u08 data)
{
	SPI_SELECT_ETHERNET;
  spiPut(WCR | address);
  spiPut(data);
	SPI_UNSELECT_ETHERNET;
}

/******************************************************************************
 * Function:        void encBFSReg(u08 address, u08 data)
 * PreCondition:    Bank must point ot the right bank
 * Input:           address and data field
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Set the data field in the address register
 * Note:            None
 *****************************************************************************/
static void encBFSReg(u08 address, u08 data)
{
	SPI_SELECT_ETHERNET;
  spiPut(BFS | address);
  spiPut(data);
	SPI_UNSELECT_ETHERNET;
}

/******************************************************************************
 * Function:        void encBFCReg(u08 address, u08 data)
 * PreCondition:    Bank must point ot the right bank
 * Input:           address and data fiels
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Clear the data field in the address register
 * Note:            None
 *****************************************************************************/
static void encBFCReg(u08 address, u08 data)
{
	SPI_SELECT_ETHERNET;
  spiPut(BFC | address);
  spiPut(data);
	SPI_UNSELECT_ETHERNET;
}

/******************************************************************************
 * Function:        void encWriteReg16(u08 address, u16 data)
 * PreCondition:    Bank must point ot the right bank
 * Input:           address and data to write
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Send value at the address over the SPI bus
 * Note:            None
 *****************************************************************************/
static void encWriteReg16(u08 address, u16 data)
{
	encWriteReg(address,   (u08) (data & 0xff));
 	encWriteReg(address+1, (u08) (data >> 8));
}

/******************************************************************************
 * Function:        void encWritePHYReg(u08 address, u16 data)
 * PreCondition:    None
 * Input:           address and data to write
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Send value at the address over the SPI bus
 * Note:            None
 *****************************************************************************/
static void encWritePHYReg(u08 address, u16 data)
{
	/* Switch to Bank 2 */
	encBankSelect(BANK2);
  /* Write the PHY address to write */
	encWriteReg(MIREGADR, address);
	
	/* Write the data */
  encWriteReg16(MIWRL, data);

	/* Switch to Bank 3 */
	encBankSelect(BANK3);
	while( (encReadMacReg(MISTAT) & MISTAT_BUSY) );
}

/******************************************************************************
 * Function:        u16 encReadPHYReg(u08 address)
 * PreCondition:    None
 * Input:           address to read from
 * Output:          data from a PHY register
 * Side Effects:    None
 *
 * Overview:        Read a value from a PHY register
 * Note:            None
 *****************************************************************************/
static u16 encReadPHYReg(u08 address)
{
  u16 value;

	/* Switch to Bank 2 */
	encBankSelect(BANK2);
  /* Write the PHY address to read */
	encWriteReg(MIREGADR, address);
  /* Send the read command */
  encWriteReg(MICMD, MICMD_MIIRD);

	/* Wait end of command */
	/* Switch to Bank 3 */
	encBankSelect(BANK3);
  while( (( encReadMacReg(MISTAT)) & MISTAT_BUSY) );

  /* Clear the read command */
	/* Switch to Bank 2 */
	encBankSelect(BANK2);
  encWriteReg(MICMD, 0);

  value = encReadMacReg(MIRDH) << 8;
  value += encReadMacReg(MIRDL);
  return value;
}

/******************************************************************************
 * Function:        void encBankSelect(u08 bank)
 * PreCondition:    None
 * Input:           bank (0, 1, 2 or 3)
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Set the bank select 
 * Note:            None
 *****************************************************************************/
static void  encBankSelect(u08 bank)
{
	/* Start by clearing the bank select bits */
  encBFCReg(ECON1, ECON1_BSEL1 | ECON1_BSEL0);
	encBFSReg(ECON1, bank);
}

/******************************************************************************
 * Function:        void encMACwrite(u08 data)
 * PreCondition:    EWRPT must point to the location to be written to
 * Input:           data to write
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Write a byte in the MAC memory with the intention
 *                  of sending a frame later 
 * Note:            None
 *****************************************************************************/
static void encMACwrite(u08 data)
{
  SPI_SELECT_ETHERNET;
  spiPut(WBM);              	
  spiPut(data);
	SPI_UNSELECT_ETHERNET;
}

/******************************************************************************
 * Function:        void encMACwriteBulk(u08 *buffer, u16 length)
 * PreCondition:    EWRPT must point to the location to be written to
 * Input:           data to write and length of data
 * Output:          None
 * Side Effects:    None
 *
 * Overview:        Multi Write in the MAC memory with the intention
 *                  of sending a frame later 
 * Note:            None
 *****************************************************************************/
static void encMACwriteBulk(u08 *buffer, u16 length)
{
  SPI_SELECT_ETHERNET;
  spiPut(WBM);              	
  while (length--)
  {
    spiPut(*buffer++);
  }
	SPI_UNSELECT_ETHERNET;
}

/******************************************************************************
 * Function:        u08 data encMACread(void)
 * PreCondition:    ERDPT must point to the location to read from
 * Input:           None
 * Output:          Data read
 * Side Effects:    None
 *
 * Overview:        Read a byte from the MAC memory
 * Note:            None
 *****************************************************************************/
static u08 encMACread(void)
{
  u08 value;

  SPI_SELECT_ETHERNET;
  spiPut(RBM);
  value = spiPut(0x00);							
	SPI_UNSELECT_ETHERNET;
  return value;
}

/******************************************************************************
 * Function:        encMACreadBulk(u08 *buffer, u16 length)
 * PreCondition:    ERDPT must point to the location to read from
 * Input:           Buffer to put the data and length of data to read
 * Output:          Data read in the buffer
 * Side Effects:    None
 *
 * Overview:        Read multiple bytes from the MAC memory
 * Note:            None
 *****************************************************************************/
static void encMACreadBulk(u08 *buffer, u16 length)
{
  SPI_SELECT_ETHERNET;
  spiPut(RBM);
  while (length--)
  {
    *buffer++ = spiPut(0x00);
  }
	SPI_UNSELECT_ETHERNET;
}


