/*! \file mmc.c \brief MultiMedia and SD Flash Card Interface. */
//*****************************************************************************
//
// File Name	: 'mmc.c'
// Title		: MultiMedia and SD Flash Card Interface
// Author		: Pascal Stang - Copyright (C) 2004
// Created		: 2004.09.22
// Revised		: 2004.09.22
// Version		: 0.1
// Target MCU	: Atmel AVR Series
// Editor Tabs	: 4
//
// NOTE: This code is currently below version 1.0, and therefore is considered
// to be lacking in some functionality or documentation, or may not be fully
// tested.  Nonetheless, you can expect most functions to work.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

//----- Include Files ---------------------------------------------------------

//#include <avr/io.h>			// include I/O definitions (port names, pin names, etc)
//##include <avr/signal.h>		// include "signal" names (interrupt names)
//#include <avr/interrupt.h>	// include interrupt support



#include "mmc.h"

// include project-specific hardware configuration
#include "mmcconf.h"
#include "spiLPC.h"
#include "config.h"

// Global variables

// Functions
//void mmcCShigh(void) {
//	IOSET0 = 0x00800000;
//}

//void mmcCSlow(void) {
//	IOCLR0 = 0x00800000;
//}


u08 mmcReset(void)
{
	u08 retry;
	u08 r1=0;
	u08 i;

	retry = 0;
	do
	{
		// send dummy bytes with CS high before accessing
		for (i=0; i!=10; i++)
    {
	    spiPut(0xff);	
		}
		// resetting card, go to SPI mode
		r1 = mmcSendCommand(MMC_GO_IDLE_STATE, 0);
		#ifdef MMC_DEBUG
		printf("MMC_GO_IDLE_STATE: R1=0x%02x\r\n", r1);
		#endif
		// do retry counter
		retry++;
		if(retry>10) return -1;
	} while(r1 != 0x01);

  	do
	{
		// resetting card, go to SPI mode
		r1 = mmcSendCommand(MMC_GO_IDLE_STATE, 0);
		#ifdef MMC_DEBUG
		printf("MMC_GO_IDLE_STATE: R1=0x%02x\r\n", r1);
		#endif
		// do retry counter
		retry++;
		if(retry>100) return -1;
	} while(r1 != 0x01);

	// TODO: check card parameters for voltage compliance
	// before issuing initialize command

	retry = 0;
	do
	{
		// initializing card for operation
		r1 = mmcSendCommand(MMC_SEND_OP_COND, 0);
		#ifdef MMC_DEBUG
		printf("MMC_SEND_OP_COND: R1=0x%02x\r\n", r1);
		#endif
		// do retry counter
		retry++;
		if(retry>100) return -1;
	} while(r1);
		
	// turn off CRC checking to simplify communication
	r1 = mmcSendCommand(MMC_CRC_ON_OFF, 0);
	#ifdef MMC_DEBUG
	printf("MMC_CRC_ON_OFF: R1=0x%02x\r\n", r1);
	#endif

	// set block length to 512 bytes
	r1 = mmcSendCommand(MMC_SET_BLOCKLEN, 512);
	#ifdef MMC_DEBUG
	printf("MMC_SET_BLOCKLEN: R1=0x%02x\r\n", r1);
	#endif

	// return success
	return 0;
}

u08 mmcSendCommand(u08 cmd, u32 arg)
{
	u08 r1;

	// assert chip select
	SPI_SELECT_MMC;
	// issue the command
	r1 = mmcCommand(cmd, arg);
	// release chip select
	SPI_UNSELECT_MMC;

	return r1;
}

u08 mmcRead(u32 sector, u08* buffer)
{
	u08 r1;
	u16 i;

	// assert chip select
	SPI_SELECT_MMC;
	// issue command
	r1 = mmcCommand(MMC_READ_SINGLE_BLOCK, sector<<9);
	#ifdef MMC_DEBUG
	printf("MMC Read Block Sector=%04x R1=%02x\r\n", sector, r1);
	#endif
	// check for valid response
	if(r1 != 0x00)
		return r1;
	// wait for block start
	while( spiPut(0xff) != MMC_STARTBLOCK_READ);
	// read in data
	for(i=0; i<0x200; i++)
	{
		*buffer++ = spiPut(0xff);
	}
	// read 16-bit CRC
	spiPut(0xff);
	spiPut(0xff);
	// release chip select
	SPI_UNSELECT_MMC;
	// return success
	return 0;
}

u08 mmcWrite(u32 sector, u08* buffer)
{
	u08 r1;
	u16 i;

	// assert chip select
	SPI_SELECT_MMC;
	// issue command
	r1 = mmcCommand(MMC_WRITE_BLOCK, sector<<9);
	#ifdef MMC_DEBUG
	printf("MMC Write Block R1=0x%x\r\n", r1);
	#endif
	// check for valid response
	if(r1 != 0x00)
		return r1;
	// send dummy
  spiPut(0xff);
	// send data start token
	spiPut(MMC_STARTBLOCK_WRITE);
	// write data
	for(i=0; i<0x200; i++)
	{
		spiPut(*buffer++);
	}
	// write 16-bit CRC (dummy values)
	spiPut(0xff);
	spiPut(0xff);
	// read data response token
	r1 = spiPut(0xff);
	if( (r1&MMC_DR_MASK) != MMC_DR_ACCEPT)
		return r1;
	#ifdef MMC_DEBUG
	printf("Data Response Token=0x%x\r\n", r1);
	#endif
	// wait until card not busy
	while( !spiPut(0xff) );
	// release chip select
	SPI_UNSELECT_MMC;
	// return success
	return 0;
}

u08 mmcCommand(u08 cmd, u32 arg)
{
	u08 r1;
	u08 retry=0;
	// send command
	spiPut(cmd | 0x40);
	spiPut(arg>>24);
	spiPut(arg>>16);
	spiPut(arg>>8);
	spiPut(arg);
	spiPut(0x95);	// crc valid only for MMC_GO_IDLE_STATE
	// end command
	// wait for response
	// if more than 8 retries, card has timed-out
	// return the received 0xFF
	while((r1 = spiPut(0xff)) == 0xff)
		if(retry++ > 8) break;
	// return response
	return r1;
}


